diff roundup/backends/rdbms_common.py @ 5525:bb7865241f8a

Make CSV import/export compatible across Python versions (also RDBMS journals) (issue 2550976, issue 2550975). The roundup-admin export and import commands are used for migrating between different database backends. It is desirable that they should be usable also for migrations between Python 2 and Python 3, and in some cases (e.g. with the anydbm backend) this may be required. To be usable for such migrations, the format of the generated CSV files needs to be stable, meaning the same as currently used with Python 2. The export process uses repr() to produce the fields in the CSV files and eval() to convert them back to Python data structures. repr() of strings with non-ASCII characters produces different results for Python 2 and Python 3. This patch adds repr_export and eval_import functions to roundup/anypy/strings.py which provide the required operations that are just repr() and eval() in Python 2, but are more complicated in Python 3 to use data representations compatible with Python 2. These functions are then used in the required places for export and import. repr() and eval() are also used in storing the dict of changed values in the journal for the RDBMS backends. It is similarly desirable that the database be compatible between Python 2 and Python 3, so that export and import do not need to be used for a migration between Python versions for non-anydbm back ends. Thus, this patch changes rdbms_common.py in the places involved in storing journals in the database, not just in those involved in import/export. Given this patch, import/export with non-ASCII characters appear based on some limited testing to work across Python versions, and an instance using the sqlite backend appears to be compatible between Python versions without needing import/export, *if* the sessions/otks databases (which use anydbm) are deleted when changing Python version.
author Joseph Myers <jsm@polyomino.org.uk>
date Sun, 02 Sep 2018 23:48:04 +0000
parents 6b0c542642be
children 1a0498c1ed90
line wrap: on
line diff
--- a/roundup/backends/rdbms_common.py	Sun Sep 02 18:59:41 2018 +0000
+++ b/roundup/backends/rdbms_common.py	Sun Sep 02 23:48:04 2018 +0000
@@ -69,7 +69,7 @@
 from roundup.date import Range
 
 from roundup.backends.back_anydbm import compile_expression
-from roundup.anypy.strings import b2s, bs2b, us2s
+from roundup.anypy.strings import b2s, bs2b, us2s, repr_export, eval_import
 
 
 # dummy value meaning "argument not passed"
@@ -1302,7 +1302,7 @@
         if isinstance(params, type({})):
             self._journal_marshal(params, classname)
 
-        params = repr(params)
+        params = repr_export(params)
 
         dc = self.to_sql_value(hyperdb.Date)
         journaldate = dc(journaldate)
@@ -1328,7 +1328,7 @@
             # make the journalled data marshallable
             if isinstance(params, type({})):
                 self._journal_marshal(params, classname)
-            params = repr(params)
+            params = repr_export(params)
 
             self.save_journal(classname, cols, nodeid, dc(journaldate),
                 journaltag, action, params)
@@ -1366,7 +1366,7 @@
         res = []
         properties = self.getclass(classname).getprops()
         for nodeid, date_stamp, user, action, params in journal:
-            params = eval(params)
+            params = eval_import(params)
             if isinstance(params, type({})):
                 for param, value in params.items():
                     if not value:
@@ -2892,8 +2892,8 @@
                 value = value.get_tuple()
             elif isinstance(proptype, hyperdb.Password):
                 value = str(value)
-            l.append(repr(value))
-        l.append(repr(self.is_retired(nodeid)))
+            l.append(repr_export(value))
+        l.append(repr_export(self.is_retired(nodeid)))
         return l
 
     def import_list(self, propnames, proplist):
@@ -2914,10 +2914,11 @@
         if not "id" in propnames:
             newid = self.db.newid(self.classname)
         else:
-            newid = eval(proplist[propnames.index("id")])
+            newid = eval_import(proplist[propnames.index("id")])
         for i in range(len(propnames)):
-            # Use eval to reverse the repr() used to output the CSV
-            value = eval(proplist[i])
+            # Use eval_import to reverse the repr_export() used to
+            # output the CSV
+            value = eval_import(proplist[i])
 
             # Figure the property for this column
             propname = propnames[i]
@@ -3010,7 +3011,7 @@
                     # old tracker with data stored in the create!
                     params = {}
                 l = [nodeid, date, user, action, params]
-                r.append(list(map(repr, l)))
+                r.append(list(map(repr_export, l)))
         return r
 
 class FileClass(hyperdb.FileClass, Class):

Roundup Issue Tracker: http://roundup-tracker.org/